/* PHYSCOLO.C -- Program to display the physical screen colors */
/* Version 0.91 */
/* This program is Copyright (c) 1992 by Raja Thiagarajan. However, you
   may use it for any non-commercial purpose. */

#define INCL_GPI
#define INCL_WIN
#include <os2.h>

MRESULT EXPENTRY ClientWinProc (HWND hWnd, USHORT msg, MPARAM mp1, MPARAM mp2);

static LONG hasPalMan;      /* Whether the Palette Manager is available */
static LONG colorsToShow;   /* How many colored rectangles to draw */

static HPAL hpal;           /* Handle to the palette we'll use */

INT main (VOID)
{
   HAB   hab;         /* The usual bunch of variables for PM programs */
   HMQ   hMQ;
   QMSG  qmsg;
   HWND  hWnd,
         hWndClient;

   ULONG createFlags  =  FCF_TITLEBAR | FCF_SYSMENU | FCF_SIZEBORDER
                                      | FCF_MINMAX | FCF_SHELLPOSITION
                                      |  FCF_TASKLIST;


#define clientClass "physColo"

   {
      LONG sColors; /* How many colors are available */
      HPS hpsScr;   /* These temporary vars are */
      HDC hdcScr;   /*   used to query the screen driver */
      hpsScr = WinGetPS (HWND_DESKTOP);
      hdcScr = GpiQueryDevice (hpsScr);
      DevQueryCaps (hdcScr, CAPS_COLORS, 1L, &sColors);
      DevQueryCaps (hdcScr, CAPS_ADDITIONAL_GRAPHICS, 1, &hasPalMan);
      hasPalMan &= CAPS_PALETTE_MANAGER;
      colorsToShow = sColors;
#define MAX_PAL_SIZE 256
      /* To save time, only draw the first 256 colors if there are more */
      if (colorsToShow > MAX_PAL_SIZE)
        colorsToShow = MAX_PAL_SIZE;
   }

   hab = WinInitialize (0);  /* initialize PM usage */

   if (hasPalMan) {
      /* Create the palette. Since we use PC_EXPLICIT, table entries are
         interpreted as palette register numbers, rather than RGB values */
      ULONG  tbl [MAX_PAL_SIZE];
      INT    j;
      for (j = 0; j < colorsToShow; j++) {
         tbl [j] = PC_EXPLICIT * 16777216 + j;
      }
      hpal = GpiCreatePalette (hab, 0L, LCOLF_CONSECRGB, colorsToShow, tbl);
   }

   hMQ = WinCreateMsgQueue (hab, 0);   /* create message queue */

   WinRegisterClass (hab, clientClass, (PFNWP) ClientWinProc, CS_SIZEREDRAW, 0);

      /* Create standard window and client. Note that the title message
         depends on whether the Palette Manager is available or not */
   if (hasPalMan) {
      hWnd = WinCreateStdWindow (HWND_DESKTOP, WS_VISIBLE, &createFlags,
                                 clientClass, "Physical Color Viewer", 0L,
                                 0UL, 0, &hWndClient);
   } else {
      hWnd = WinCreateStdWindow (HWND_DESKTOP, WS_VISIBLE, &createFlags,
                                 clientClass, "Color Viewer", 0L, 0UL, 0,
                                 &hWndClient);
   }

   while (WinGetMsg (hab, &qmsg, NULLHANDLE, 0, 0)) /* messages dispatch loop */
      WinDispatchMsg (hab, &qmsg);

   if (hasPalMan) {
      GpiDeletePalette (hpal);
   }

   WinDestroyWindow (hWnd);           /* destroy frame window */
   WinDestroyMsgQueue (hMQ);          /* destroy message queue */
   WinTerminate (hab);                /* terminate PM usage */

   return 0;
}

MRESULT EXPENTRY ClientWinProc (HWND hWnd, USHORT msg, MPARAM mp1, MPARAM mp2)
{
   static USHORT cWid, cHi;

   switch (msg)
      {
         case WM_ERASEBACKGROUND:
            return (MRESULT) TRUE;
         case WM_PAINT:
            {
               HPS    hps;
               POINTL ptl;
               LONG   j;

               hps = WinBeginPaint (hWnd, NULLHANDLE, NULL);
               if (hasPalMan) {
                  ULONG palSize = colorsToShow;
                  GpiSelectPalette (hps, hpal);
                  WinRealizePalette (hWnd, hps, &palSize);
               }
               for (j = 0; j < colorsToShow; j++) {
                  GpiSetColor (hps, j);
                  ptl.y = 0;
                  ptl.x = j * cWid / colorsToShow;
                  GpiMove (hps, &ptl);
                  ptl.y = cHi;
                  ptl.x += cWid / colorsToShow;
                  GpiBox (hps, DRO_FILL, &ptl, 0L, 0L);
               }
               WinEndPaint (hps);
               return (MRESULT) FALSE;
            }
         case WM_SIZE:
            cWid = SHORT1FROMMP (mp2);
            cHi = SHORT2FROMMP (mp2);
            return (MRESULT) FALSE;
         default:
            return (WinDefWindowProc (hWnd, msg, mp1, mp2));
            break;
      }  /*end switch*/
   return (MRESULT) FALSE;
}
